;;;; $Id: plugin-age.lisp,v 1.3 2004/02/19 18:10:59 boelthorn Exp $
(def-xchat-plugin "Age Computer"
    :version "0.1"
    :package-name "PLUGIN-AGE"
    :description "Calculates your age in years")

;;; This plugin provides two basic commands:
;;; 1. /calcage day month year   - Calculates the age and posts it in the 
;;;                                current channel
;;; 2. /calcbirth years          - Calculates the birth date and posts it
;;;                                in the current channel

(defun calc-age (day month year &optional (hour 12) (minutes 00))
  (/ (- (get-universal-time) (encode-universal-time 0 minutes hour day month year))
     (* 60 60 24 3652425/10000)))

(defun calc-birth (years)
  (let ((seconds (round (* years 3652425/10000 24 60 60))))
    (multiple-value-bind (second minute hour date month year)
	(decode-universal-time 
	 (- (get-universal-time) seconds))
      (declare (ignore second minute hour))
      (values date month year))))

(defun cbAge (word word_eol)
  (let ((day (parse-integer (second word)))
	(month (parse-integer (third word)))
	(year (parse-integer (fourth word))) )
    (%commandf "ACTION computes: If you are born ~A.~A.~A , you are ~F years old."
	       day month year (calc-age day month year)))
  +xchat-eat-all+)

(defun parse-digit (char)
  (if (<= (char-code #\0) (char-code char) (char-code #\9))
      (- (char-code char) (char-code #\0))
      (error "~S is no digit." char)))

(defun parse-fraction (str)
  (loop with past-point = nil
	and  dec-places = 0
	and  value = 0
        for x across (string-trim '(#\Space #\Tab) str)
	do (cond
	     ((char= x #\.) (setq past-point t))
	     (t (if past-point (incf dec-places))
		(setf value (+ (* 10 value)
			       (parse-digit x)))))
	finally (return (/ value (expt 10 dec-places)))))
	
(defun cbBirth (word word_eol)
  (let ((age (parse-fraction (second word))))
    (multiple-value-bind (date month year)
	(calc-birth age)
      (%commandf "ACTION calculates that if you are ~A years old, you must have been born on ~A.~A.~A."
		(second word) date month year)))
  +xchat-eat-all+)
  

(%hook-command *plugin* "calcage" #'cbAge "Calculates your age: /calcage day month year")
(%hook-command *plugin* "calcbirth" #'cbBirth "Calculates the birth date: /calcbirth age")