;;; $Id: plugin-xmms.lisp,v 1.3 2004/02/19 18:10:59 boelthorn Exp $
(def-xchat-plugin "XMMS InfoPipe Backend"
    :version "0.1"
    :description "Tells others what you are listening to")

;; This is actually a very simple plugin, but shows how to communicate
;; with the outside world. This plugin requires that you have
;; a certain XMMS plugin called InfoPipe installed.

(defconstant +pipe-file+ "/tmp/xmms-info")

(defun string-prefix (prefix string)
  "Checks if prefix is the ..mmh.. prefix of string and returns the rest"
  (if (string-equal prefix string :end2 (length prefix))
      (subseq string (length prefix))))

(defmacro L1 (fun-name &rest args)
  (let ((x (gensym)))
    `(lambda (,x)
      (,fun-name ,@args ,x))))

(defmacro L0 (fun-name &rest args)
  `(lambda ()
    (,fun-name ,@args)))

(defun call-until (pred fun)
  "Calls fun until pred returns non-nil. Returns this non-nil value"
  (loop for val = (funcall fun)
	for res = (funcall pred val)
	until res
	finally (return res)))

(defun check-xmms ()
  "Parses InfoPipe's output until a 'Title:' is found"
  (handler-case  
      (with-open-file (xmms-pipe +pipe-file+)
	(let ((title (call-until (L1 string-prefix "Title: ")
				 (L0 read-line xmms-pipe))))
	  (if title
	      (%commandf "ACTION is listening to: ~A" title))))
    (file-error () (format t "Error reading from XMMS InfoPipe. Do you have InfoPipe enabled?~%"))))
		  	     
(defun init-xmms ()
  (%hook-command *plugin* "MP3INFO" (lambda (word word_eol)
				      (declare (ignore word word_eol))
				      (check-xmms)
				      +xchat-eat-all+)
		 "Announces your current song."))


(init-xmms)